from ctypes import *
from functools import partial
import sys

_libchewing = None
if sys.platform == "win32": # Windows
    import os.path
    # find in current dir first
    dll_path = os.path.join(os.path.dirname(__file__), "chewing.dll")
    if not os.path.exists(dll_path):
        dll_path = "chewing.dll" # search in system path
    _libchewing = CDLL(dll_path)
else: # UNIX-like systems
    _libchewing = CDLL('libchewing.so.3')

_libchewing.chewing_commit_String.restype = c_char_p
_libchewing.chewing_buffer_String.restype = c_char_p
_libchewing.chewing_cand_String.restype = c_char_p
_libchewing.chewing_zuin_String.restype = c_char_p
_libchewing.chewing_aux_String.restype = c_char_p
_libchewing.chewing_get_KBString.restype = c_char_p
_libchewing.chewing_new.restype = c_void_p
_libchewing.chewing_new2.restype = c_void_p


def Init(datadir, userdir):
    return _libchewing.chewing_Init(datadir, userdir)


class ChewingContext:
    def __init__(self, **kwargs):
        if not kwargs:
            self.ctx = _libchewing.chewing_new()
        else:
            syspath = kwargs.get("syspath", None)
            userpath = kwargs.get("userpath", None)
            self.ctx = _libchewing.chewing_new2(
                syspath,
                userpath,
                None,
                None)

    def __del__(self):
        _libchewing.chewing_delete(c_void_p(self.ctx))

    def __getattr__(self, name):
        func = 'chewing_' + name
        if hasattr(_libchewing, func):
            wrap = partial(getattr(_libchewing, func), c_void_p(self.ctx))
            setattr(self, name, wrap)
            return wrap
        else:
            raise AttributeError(name)

    def Configure(self, cpp, maxlen, direction, space, kbtype):
        self.set_candPerPage(cpp)
        self.set_maxChiSymbolLen(maxlen)
        self.set_addPhraseDirection(direction)
        self.set_spaceAsSelection(space)
        self.set_KBType(kbtype)



#!/usr/bin/env python
import os
import subprocess
import re
import random

def show_command_to_reproduce(keystrokes):
    print '''echo '%s' | ./testchewing''' % keystrokes.strip()

def genkey(seed, length=100):
    p = subprocess.Popen(
            ['./randkeystroke', '-r', '-s', str(seed), '-n', str(length)],
            stdout=subprocess.PIPE)
    return p.communicate()[0]

def reset_state():
    if os.path.exists('uhash.dat'):
        os.remove('uhash.dat')

def run(keystrokes):
    reset_state()

    p = subprocess.Popen(
            './testchewing',
            stdin=subprocess.PIPE,
            stdout=subprocess.PIPE)
    p.communicate(keystrokes)
    return p.returncode

def parse_keys(keystrokes):
    return re.findall(r'<[^<>]+>|[^<>]', keystrokes)

def try_to_reduce_key(keystrokes):
    keys = parse_keys(keystrokes)

    while True:
        for i in range(len(keys)):
            keys2 = keys[0:i] + keys[i+1:]
            keystrokes = ''.join(keys2)
            if run(keystrokes) != 0:
                keys = keys2
                #print len(keys2)
                #show_command_to_reproduce(keystrokes)
                break
        else:
            show_command_to_reproduce(keystrokes)
            return

def main():
    for seed in xrange(10**9):
        # show progress
        if seed % 1000 == 0:
            print 'seed', seed

        keystroke = genkey(seed)
        if run(keystroke) != 0:
            print 'failed for seed', seed
            show_command_to_reproduce(keystroke)
            try_to_reduce_key(keystroke)
            #break



if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import chewing

chewing.Init('/usr/share/chewing', '/tmp')
ctx = chewing.ChewingContext()
ctx.Configure(18, 16, 0, 1, 0)
ctx.set_ChiEngMode(1)
ctx.handle_Default(ord("g"))
ctx.handle_Default(ord("j"))
ctx.handle_Space()
print ctx.buffer_String()
ctx.handle_Default(ord("b"))
ctx.handle_Default(ord("j"))
ctx.handle_Default(ord("4"))
ctx.handle_Default(ord("z"))
ctx.handle_Default(ord("8"))
ctx.handle_Default(ord("3"))
ctx.handle_Enter()
print ctx.commit_String()
ctx = None



#!/usr/bin/env python3
import argparse
import os
import sys


def get_args():
    parser = argparse.ArgumentParser(
        description='Create keystroke from text.')
    parser.add_argument('input', metavar='input', type=str, nargs=1,
        help='Input Chinese text file')
    parser.add_argument('output', metavar='output', type=str, nargs=1,
        help='Output keystroke file')
    parser.add_argument('phone', metavar='phone', type=str, nargs='?',
        default=os.path.join(os.path.dirname(sys.argv[0]), '..', 'data', 'phone.cin'),
        help='phone.cin')
    return parser.parse_args()


def read_phone(phone):
    phone_table = {}
    with open(phone) as f:
        for l in f:
            if l.startswith('%chardef  begin'):
                break
        for l in f:
            if l.startswith('%chardef  end'):
                break
            item = l.split()
            phone_table[item[1]] = item[0]
    return phone_table


def main():
    args = get_args()
    phone_table = read_phone(args.phone)

    in_file = args.input[0]
    out_file = args.output[0]
    word_count = 0

    with open(in_file) as in_, open(out_file, "w") as out_:
        for l in in_:
            for c in l:
                if c in phone_table:
                    out_.write(phone_table[c])
                    word_count += 1
        out_.write('\n')
    print('word count = {}'.format(word_count))


if __name__ == '__main__':
    main()
